package ksu.cis.collaboration;

import java.util.*;
import java.net.*;
import java.io.*;
import ksu.cis.mom.*;
import ksu.cis.collaboration.gui.*;

/**
 * This type was created in VisualAge.
 */
public class ServerAgent extends Agent
{
	ServerAgentGUI gui = new ServerAgentGUI();

	private Vector infoSources = new Vector(25);
	private Vector infoSeekers = new Vector(25);
/**
 * ServerAgent constructor comment.
 */
public ServerAgent(String n, int p)
{
	super(n, p);
	
	/* server initialization */

	MessageHandler handler;
	handler = new MessageHandler(port, this);
	handler.start();

}
/**
 * This method was created in VisualAge.
 * @param agent java.lang.String
 * @param service java.lang.String
 * @param host java.lang.String
 * @param port int
 */
protected void add_info_source(String infoType, String agent, String host, int port)
{
	
	this.infoSources.addElement(new Registration(agent, host, port, infoType));

	// check to see if anyone has requested this information
	Source s = find_seeker(infoType);
	if (s != null)
	{
		write("Found seeker for infotype "+ infoType + " on host " + s.host + " at port " + s.port);
		Thread init = new Thread(new Server_Return_Info_Source(this, s.host, s.port, host, port, infoType));		
		init.start();
	}
}
/**
 * This method was created in VisualAge.
 * @param agent java.lang.String
 * @param service java.lang.String
 * @param host java.lang.String
 * @param port int
 */
protected void add_info_stream(String infoType, String agent, String host, int port)
{
	this.infoSeekers.addElement(new Registration(agent, host, port, infoType));

	// check to see if anyone has registered this information
	Source s = find_source(infoType);
	if (s != null)
	{
		write("Found source for infotype " + infoType + " - " + s.agentName + " on host " + s.host + " at port " + s.port);
		Thread init = new Thread(new Server_Return_Info_Source(this, host, port, s.host, s.port, infoType));
		init.start();
	}
}
/**
 * This method was created in VisualAge.
 * @param agent java.lang.String
 * @param service java.lang.String
 * @param host java.lang.String
 * @param port int
 */
protected Source find_seeker(String infoType)
{
	java.util.Enumeration enum = this.infoSeekers.elements();
	Registration v;
	while (enum.hasMoreElements())
	{
		v = (Registration) enum.nextElement();
		if (v.service.equals(infoType))
			return new Source(v.agent, v.host, v.port);
	}
	return null;
}
/**
 * This method was created in VisualAge.
 * @param agent java.lang.String
 * @param service java.lang.String
 * @param host java.lang.String
 * @param port int
 */
protected Source find_source(String infoType)
{
	java.util.Enumeration enum = this.infoSources.elements();
	Registration v;
	while (enum.hasMoreElements())
	{
		v = (Registration) enum.nextElement();
		if (v.service.equals(infoType))
			return new Source(v.agent, v.host, v.port);
	}
	return null;
}
/**
 * This method was created in VisualAge.
 * @param args java.lang.String[]
 */
public static void main(String[] args)
{
	ServerAgent s = new ServerAgent("Server", 5001);
	s.run();
}
/**
 * This method was created in VisualAge.
 */
protected void printInfoSeekers()
{
	java.util.Enumeration enum = this.infoSeekers.elements();
	Registration v;
	enum = this.infoSeekers.elements();
	while (enum.hasMoreElements())
	{
		v = (Registration) enum.nextElement();
		write("  [" + v.agent + ", " + v.service + ", " + v.host + ", " + v.port + "]");
	}
}
/**
 * This method was created in VisualAge.
 */
protected void printInfoSources()
{
	java.util.Enumeration enum = this.infoSources.elements();
	Registration v;
	enum = this.infoSources.elements();
	while (enum.hasMoreElements())
	{
		v = (Registration) enum.nextElement();
		write("< " + v.agent + ", " + v.service + ", " + v.host + ", " + v.port + ">");
	}
}
/**
 * receiveMessage method comment.
 */

public void receiveMessage(Socket server, ObjectInputStream input, ObjectOutputStream output)
{
	int i;
	Message m;
	Thread t;

	/* define all possible conversations receivable here */

	try
	{
		m = (Message) input.readObject();
		write("Received message " + m.performative + " from " + m.sender);
		if (m.performative.equals("register"))
		{
			t = new Thread(new Server_Register_Info_Source(server, input, output, this, m));
			t.start(); // start new thread
		} else
			if (m.performative.equals("deregister"))
			{
				t = new Thread(new Server_Deregister_Info_Source(server, input, output, this, m));
				t.start(); // start new thread
			} else
				if (m.performative.equals("register_all"))
				{
					t = new Thread(new Server_Register_For_Info(server, input, output, this, m));
					t.start(); // start new thread
				} else
					if (m.performative.equals("deregister_all"))
					{
						t = new Thread(new Server_Deregister_For_Info(server, input, output, this, m));
						t.start(); // start new thread
					} else
						if (m.performative.equals("request_source"))
						{
							t = new Thread(new Server_Request_Info_Source(server, input, output, this, m));
							t.start(); // start new thread
						} else
						{
							write(" ** Invalid Attempt to start new conversation with performative " + m.performative + " from " + m.sender);
							t = new Thread(new Sorry(server, input, output, this, m));
							t.start(); // start new thread
						}
	} catch (ClassNotFoundException cnfex)
	{
		write(" ** ClassNotFoundException ");
	} catch (IOException cnfex)
	{
		write(" ** IOException on port (ServerAgent.receiveMessage)");
	}
}
/**
 * This method was created in VisualAge.
 * @param agent java.lang.String
 * @param service java.lang.String
 * @param host java.lang.String
 * @param port int
 */
protected void remove_info_seekers(String infoType, String agent, String host, int port)
{
	java.util.Enumeration enum = this.infoSeekers.elements();
	Registration v;
	while (enum.hasMoreElements())
	{
		v = (Registration) enum.nextElement();
		if (v.agent.equals(agent) && v.service.equals(infoType) 
			&& v.host.equals(host) && v.port == port)
			this.infoSeekers.removeElement(v);
	}
}
/**
 * This method was created in VisualAge.
 * @param agent java.lang.String
 * @param service java.lang.String
 * @param host java.lang.String
 * @param port int
 */
protected void remove_info_source(String infoType, String agent, String host, int port)
{
	java.util.Enumeration enum = this.infoSources.elements();
	Registration v;
	while (enum.hasMoreElements())
	{
		v = (Registration) enum.nextElement();
		if (v.agent.equals(agent) && v.service.equals(infoType) 
			&& v.host.equals(host) && v.port == port)
			this.infoSources.removeElement(v);
	}
}
/**
 * This method was created in VisualAge.
 * @param args java.lang.String[]
 */
public void run()
{
	write("Starting Server \n");
}
/**
 * This method was created in VisualAge.
 */
public void write(String s) {
 	gui.write(s);
	///System.out.println(s);
}
}
