/*
 * @(#)ServerAgent.java
 * FileSharing package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */
package ksu.cis.example.FileSharing;

import java.io.*;
import java.net.*;
import java.util.*;
import ksu.cis.mom.*;

/**
 * @author Chairoj Mekprasertvit
 *
 */
public class ServerAgent extends Agent
{
  public static final int multicastPort[] = {3002, 3003};
  public static final int secureMulticastPort[] = {0,0};
  public static final int unicastPort = 3000;

  public ServerAgent()
  {
    super("ServerAgent", unicastPort, multicastPort, 0, 0, secureMulticastPort);
    try
    {
      super.group = new InetAddress[1];
      super.group[0] = InetAddress.getByName("239.192.100.100");
    }
    catch (Throwable e)
    {
      write("Invalid Internet Address");
    }
    
    MessageHandler h = new MessageHandler(unicastPort, this);
    h.start();
    MulticastHandler mh = new MulticastHandler(this, multicastPort[0], 1, group[0]);
    mh.start();
  }

  /**
   * 
   */
  public void receiveMessage(
    Socket server,
    ObjectInputStream input,
    ObjectOutputStream output)
  {
    Thread t;
    Message m;
    try
    {
      m = (Message) input.readObject();
      // send accept request
      if(m.performative.equals("find file"))
      {
        ServerRespondRequest srr = new ServerRespondRequest(
          server, input, output, this, m);
        t = new Thread(srr);
        t.start();
        
        //wait until conversation is done
        while(srr.getDone())
        {
          try
          {
            Thread.sleep(1000);
          }
          catch(InterruptedException iex)
          {
            write("** ServerRespondRequest InterruptedException.");
          }
        }
        
        // check if this agent has the file, then send back
        this.write("The file is " + m.content);
        boolean found = this.findFile((String) m.content);
        if(found)
        {
          ServerReturnFile srf = new ServerReturnFile(
            this, server.getInetAddress().getHostName(), (String)m.content);
          t = new Thread(srf);
          t.start();         
        }
        // if not found, then ask the group who has it
        else
        {
          String client = server.getInetAddress().getHostName();
          ServerMulticast sm = new ServerMulticast(
            this, multicastPort[0], group[0], (String)m.getContent(), client);
          t = new Thread(sm);
          t.start();
          
          while(sm.getDone())
          {
            try
            {
              Thread.sleep(1000);
            }
            catch(InterruptedException iex)
            {
              write("** ServerRespondRequest InterruptedException.");
            }
          }          
        }
      }// end if
      else
        write("** Unrecognized Performative - IGNORED ");
    }
    catch(ClassNotFoundException cnfex)
    {
      System.out.println("** Class not found exception in " + this.name);
    }
    catch(IOException ioex)
    {
      System.out.println("** IOException in " + this.name);
    }
  }
  
  /**
   * 
   */
  public void receiveMulticastConversation(
    MulticastSocket mSocket,
    Message m,
    Vector multicast_queue)
  {
    Thread t;
    if (m.performative.equals("find file"))
    {
      // check if this agent has the file, then send back
      InfoObject info = (InfoObject) m.content;
      boolean found = this.findFile(info.getFileName());
      if (found)
      {
        ServerReturnFile srf =
          new ServerReturnFile(this, info.getClient(), info.getFileName());
        t = new Thread(srf);
        t.start();
      }
      else
        write("** File not found on this agent.");
    }
    else
      write("** Unrecognized Performative - IGNORED ");
  }
  
  public boolean findFile(String fileName)
  {
    String user_dir = System.getProperty ("user.dir");

    File file = new File(user_dir);
    String all[] = file.list();
    boolean found = false;
    
    for(int i=0; i<all.length; i++)
    {
      if(fileName.equals(all[i]))
      {
        found = true;
        write("** The file is found.");
        break;
      }
      else
        found = false;
    }
    return found;
  }

  /**
   * 
   */
  public void run()
  {
    boolean alive = true;
    while(alive)
    {
      try
      {
        Thread.sleep(1000);
      }
      catch(InterruptedException iex)
      {
        System.out.println("** Thread Interrupted in " + this.name);
      }
    }
  }
  
  public static void main(String[] args)
  {
    ServerAgent server = new ServerAgent();
    Thread t = new Thread(server);
    t.start();
  }

}
