/*
 * @(#)ServerReturnFile.java
 * FileSharing package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */
package ksu.cis.example.FileSharing;

import ksu.cis.mom.*;
import java.net.*;
import java.io.*;

/**
 * @author Chairoj Mekprasertvit
 *
 */
public class ServerReturnFile extends Conversation
{
  private ServerAgent parent;
  private String fileName;
  private String client;
  
  public ServerReturnFile(
    ServerAgent agent, 
    String client, 
    String file)
  {
    super(agent, client, agent.port);
    parent = agent;
    fileName = file;
    this.client = client;
    parent.write("client is " + client);
  }

  /**
   * 
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    Message m = new Message();
    int state = 0;
    boolean notDone = true;
    try
    {
      parent.write("ConnectionHost is " + connectionHost);
      connection = new Socket(connectionHost, connectionPort);
      output = new ObjectOutputStream(connection.getOutputStream());
      output.flush();
      input = new ObjectInputStream(connection.getInputStream());
      while (notDone)
      {
        switch (state)
        {
          case 0 :
            m.performative = "found file";
            ByteArrayOutputStream bs = new ByteArrayOutputStream();
            FileInputStream in = new FileInputStream(fileName);
            int c;
            while ((c = in.read()) != -1)
              bs.write(c);
            byte b[] = bs.toByteArray();
            InfoObject info = new InfoObject(fileName, b);
            m.content = info;
            this.sendMessage(m, output);
            state = 1;
            break;
          case 1 :
            m = this.readMessage(input);
            if (m.performative.equals("ack"))
            {
              parent.write("** Client has received the file.");
            }
            else
              parent.write("** ERROR - did not get reply back **");
            notDone = false;
            break;
        }
      }
      input.close();
      output.close();
      connection.close();
    }
    catch (UnknownHostException e)
    {
      parent.write("** Unknown Host exception in unicast. " + e.toString());
    }
    catch (EOFException oef)
    {
      parent.write(
        "** Server terminated connection in unicast. " + oef.toString());
    }
    catch (IOException e)
    {
      parent.write("** IO Exception in unicast. " + e.toString());
    }
  }
}
