/*
 * @(#)DistributorSendKey.java
 * agentMom package
 * MSE Project: Applying Broadcast/Multicast/Secured Communication to agentMom
 * Department of Computing and Information Sciences
 * Kansas State University
 */
package ksu.cis.example.KeyDistribution;

import ksu.cis.mom.*;
import javax.net.ssl.*;
import java.net.*;
import java.security.*;
import java.io.*;

/**
 * @author Chairoj Mekprasertvit
 *
 */
public class DistributorSendKey extends SecureUnicastConversation
{
  private DistributorAgent parent;
  private Key key;
  private DistributorAgentGUI gui;
  public DistributorSendKey(
    DistributorAgent agent,
    InetAddress ip,
    int su_port,
    Key key,
    DistributorAgentGUI gui)
  {
    super(agent, ip.getHostAddress(), su_port);
    parent = agent;
    this.key = key;
    this.gui = gui;
  }

  /**
   * 
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    Message m = new Message();
    int state = 0;
    boolean notDone = true;
    try
    {
      SSLSocketFactory sslFact =
        (SSLSocketFactory) SSLSocketFactory.getDefault();
      connection = (SSLSocket) sslFact.createSocket(connectionHost, connectionPort);
      output = new ObjectOutputStream(connection.getOutputStream());
      input = new ObjectInputStream(connection.getInputStream());
      output.flush();
      while(notDone)
      {
        switch(state)
        {
          case 0:
            m.performative = "GetKey";
            m.content = key;
            sendMessage(m, output);
            gui.setDisplay(
              "Distributor sent key to " + 
              connection.getInetAddress().getHostName());
            state = 1;
            break;
          case 1:
            m = readMessage(input);
            if(m.performative.equals("ack"))
            {
              gui.setDisplay(m.sender + " has received the key.");
              notDone = false;
            }
            else 
              parent.write("** Error - did not get reply back.");
            break;
        }
      }
      input.close();
      output.close();
      connection.close();
    }
    catch (EOFException oef)
    {
      parent.write("** Server terminated connection in DistributorSendKey.");
    }
    catch (IOException e)
    {
      parent.write("** IO Exception in DistributorSendKey.");
    }
  }// end run method
}// end class DistributorSendKey
